i.icon {
  vertical-align: middle; // Firefox will place icons weird otherwise

  &:before {
    display: inline-block;
    min-width: 1em;
    margin-right: .2em;

    text-align: center;
    text-decoration: inherit;
  }
}
