<?php

// autoload_psr4.php @generated by Composer

$vendorDir = dirname(dirname(__FILE__));
$baseDir = dirname($vendorDir);

return array(
    'ipl\\Web\\' => array($vendorDir . '/ipl/web/src'),
    'ipl\\Validator\\' => array($vendorDir . '/ipl/validator/src'),
    'ipl\\Stdlib\\' => array($vendorDir . '/ipl/stdlib/src'),
    'ipl\\Sql\\' => array($vendorDir . '/ipl/sql/src'),
    'ipl\\Orm\\' => array($vendorDir . '/ipl/orm/src'),
    'ipl\\I18n\\' => array($vendorDir . '/ipl/i18n/src'),
    'ipl\\Html\\' => array($vendorDir . '/ipl/html/src'),
    'Psr\\Http\\Message\\' => array($vendorDir . '/psr/http-message/src'),
);
