<?php

namespace ipl\I18n;

use ipl\Stdlib\Contract\Translator;

/**
 * Static entrypoint for a translator instance
 */
class StaticTranslator
{
    /** @var Translator */
    public static $instance;
}
